; 
;     Meta-Environment - An environment for language prototyping.
;     Copyright (C) 2000  Stichting Mathematisch Centrum, Amsterdam, The Netherlands. 
; 
;     This program is free software; you can redistribute it and/or modify
;     it under the terms of the GNU General Public License as published by
;     the Free Software Foundation; either version 2 of the License, or
;     (at your option) any later version.
; 
;     This program is distributed in the hope that it will be useful,
;     but WITHOUT ANY WARRANTY; without even the implied warranty of
;     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;     GNU General Public License for more details.
; 
;     You should have received a copy of the GNU General Public License
;     along with this program; if not, write to the Free Software
;     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
; 
; $Id: editor.el,v 1.15 2000/03/20 21:48:13 mdejonge Exp $
;
; Toolbus interface functions for structure editor
(setq langlist ())
(setq bufferlist ())

(defun tb-init (filename)
  "Load the file filename, and return a unique buffer name to the ToolBus"
  (interactive "F")
  (find-file filename)
  (setq bufferlist (cons (buffer-name) bufferlist))
  (TBvalue (concat "file-id(" (TBstring (buffer-name)) ")"))
  ()
)

(defun tb-set-char-pos (buf pos)
  "Set the cursor to position POS"
  (set-window-buffer (selected-window) buf)
  (set-marker (point-marker t) pos)
)

(defun tb-set-msg (text)
  "Set a message in the status line"
  (display-message 'generic-msg text)
)
  
(defun tb-get-focus-text (buf start length)
  "Return the substring of buffer BUF which starts at START and is LENGTH
characters long. "
  (debug-out (concat "start: " (number-to-string start) " length: " (number-to-string length)))
  (set-window-buffer (selected-window) buf)
  (let ((buftext (buffer-substring start (+ start length) buf)))
    (debug-out buf)
    (debug-out (TBstring buf))
    (TBvalue (concat "focus-text(" (TBstring buf) "," (TBstring buftext) ")" ))
    )
  ()
 )

;; (defun tb-set-menu (menuname)
;;   "Set a top level menu with name MENUNAME"
;;   (interactive "s")
;;   (add-submenu () (list menuname ""))
;;   )

(defun tb-add-menu-item (menuname itemname)
  "Add button named ITEMNAME to menu named MENUNAME. If this button is clicked by the user, the editor will send a snd-event(menu-selected(MENUNAME,ITEMNAME,current buffer)) back to the ToolBus"
  (interactive "sMenuname:
sItemname: ")
  (set-buffer-menubar current-menubar)
  (add-menu-button (list menuname) (vector itemname (list 'TBevent (concat "menu-event" "(" (TBstring menuname) "," (TBstring itemname) "," (TBstring (buffer-name)) ")" )) t ))
  )

(defun tb-registerlang (lang)
  "Register that language lang has become available"
  (setq langlist (cons lang langlist))
)

(defun tb-unset-focus (buf)
  "Undo the changes made by a corresponding call to `tb-set-focus'"
  (debug-out buf)
  (tb-set-focus buf "None" 1 0)
  ()
)

(defun tb-set-focus (buf str start length)
  "Set the focus. BUF is the buffer where the focus should be set.
STR is the sortname of the focus. START is the starting
character, and LENGTH the length (in characters) of the focus.

This is currently implemented pretty naively, but that is because I really
don't speak any lisp. The focus area is giving a lightblue background,
thereby discarding any previous font-coloring. Somebody please improve this."
  (set-window-buffer (selected-window) buf)
  (display-message 'focus (concat "Focus sort: " str))
  (remove-text-properties 1 (+ (buffer-size) 1) '(face nil))
  (let ((l (if (> (+ start length) (buffer-size))
	       (+ (buffer-size) 1) 
	     (+ start length))))
   ;; (debug-out (buffer-size))
   ;; (debug-out l)
    (put-text-property start l 'face 'secondary-selection)
    )
  ()
  )

(defun tb-get-focus (evt count)
  "Generates a get-focus event for the ToolBus. The idea is that whenever
the user clicks somewhere, a focus is drawn containing that insertion
point.

Currently don't have a way to predict a double click. That is, when a user
double clicks, first this function is performed, and then the one for
double clicking (`tb-parse'). This is unwanted behaviour."
  (if (= 1 count)
      (let ((mark (event-closest-point evt)))
	(if (> mark (buffer-size))
	    (set 'mark (buffer-size))
	  )
	(if (is-element (buffer-name) bufferlist)
	    (TBevent (concat "get-focus(" (TBstring (buffer-name)) ","
                             (number-to-string mark) ")"))
	  )
	)
    )
  () ;; Return nil so other hooks are executed regularly
  )

(defun tb-parse (evt count)
  "Generates a parse event for the ToolBus"
  (if (= 2 count)
      (if (is-element (buffer-name) bufferlist)
	  (TBevent (concat "parse(" (TBstring (buffer-name)) ")"))
	)
    )
;;  ()
)

(defun tb-changed-event (start end oldlength)
  "Generates either a insert or a delete event to the ToolBus"
  (debug-out "In tb-changed-event")
  (debug-out (concat "start: " (number-to-string start) "end: " (number-to-string end) "oldlength: " (number-to-string oldlength)))
  (if (is-element (buffer-name) bufferlist)
      (if (= oldlength 0)
					; This is for text-insertion,
					; return insert(Bufname,Loc,Str)
	  (TBevent (concat "insert(" (TBstring (buffer-name)) "," (number-to-string start) "," (TBstring (buffer-substring start end)) ")"))

					; This is for text-deletion,
					; return delete(Bufname,Loc)
	(let ((ol oldlength))
	  (while (/= ol 0)
	    (TBevent (concat "delete(" (TBstring (buffer-name)) "," (number-to-string (+ start ol)) ")"))
	    (set 'ol (- ol 1))
	    )
	  )
   	)
    )
  ()
  )

(add-hook 'after-change-functions 'tb-changed-event)
;;(add-hook 'mouse-track-click-hook 'tb-parse)
(add-hook 'mouse-track-click-hook 'tb-get-focus)

(defun test2 ()
  (interactive)
  ;;(tb-init "editor.el")
  (tb-get-focus-text "uapdelta.rmo" 1 15962)
)

(defun test ()
  (interactive)
  (set-lenspec 8)
  (set-minmsgsize 128)
  (tb-set-focus (current-buffer) "hoi" 20 200)
)

; Aux functions
(defun is-element (obj list)
  (let ((res nil)
	(l list))
    (while (and l (not res))
      (if (eq (car l) obj)
	  (set 'res t)
	(set 'l (cdr l))
	)
      )
    res)
  )

; This is probably obsolete
(defun new-window ()
  (make-frame (name "frame1"))
  ("frame1")
  )

(defun tb-unhighlight (buf)
  (interactive "B")
  (set-window-buffer (selected-window) buf)
  (remove-text-properties 1 (buffer-size) '(focus1 nil))
)

(defun tb-highlight (buf start end)
  (interactive "B" "n" "n")
  (set-window-buffer (selected-window) buf)
  (set-text-properties start end '(focus1 t))
)
(defun start ()
  "Just for debugging"
  (interactive)
  (load-file "/ufs/kuipers/petr/toolbus/adapters/emacs-adapter/tb.el")
  (set-lenspec 8)
  (set-minmsgsize 128)
)




