/*

    Meta-Environment - An environment for language prototyping.
    Copyright (C) 2000  Stichting Mathematisch Centrum, Amsterdam, The Netherlands. 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

    $Id: Area.java,v 1.8 2000/04/15 15:18:43 mdejonge Exp $
*/
package nl.cwi.gipe.stred;
import toolbus.aterm.*;


public class Area extends ATermAppl {
  public static final String sym = "area";
  
  public Area(Start st, Length ln){
    super(the_world, sym,new ATerms(the_world, st, new ATerms(the_world, ln)));
  }

  //cast constructor
  public Area(ATermAppl t) {
    super(the_world,sym,t.getArgs());
    if (sym.equals(t.getFun())==false) {
      System.err.println("Malformed term in area constructor");
      System.exit(2);
    }
  }

    public Area setStart(int i) {
    return new Area(new Start(i),getLength());
  }
    public Area setLength(int i) {
    return new Area(getStart(),new Length(i));

    //    ATermInt ti = (ATermInt) super.getArgs().getNext().getFirst();
    //ti.setInt(i);
  } 

    public Start getStart() {
    return new Start((ATermAppl) this.getArgs().getFirst());
  }
    public Length getLength() {
    return new Length((ATermAppl) this.getArgs().getNext().getFirst());
  }

  public int start() {
   Start s = getStart();
   return s.getInt();
  }
    public int length() {
    Length l = getLength();
    return l.getInt();
  }
  
  public static Area createArea(Path p, ATerm t) {
      try {
	  return new Area(new Start(areaStart(p,t)), new 
			  Length(areaLength(Subtree.subtree(t,p))));
      }
      catch (SubtreeException e) {
	  System.err.println(e);
	  System.exit(2);
      }
      return null; // duh!
  }
  
  public static int areaStart(Path p, ATerm t) {
      return 1 + areaStartPrime(p,t);
  }
  private static int areaStartPrime(Path p, ATerm t) {
    AFun args = new AFun(the_world,"args");
    AFun elems = new AFun(the_world,"elems");
    AFun w_left = new AFun(the_world,"w-left");
    AFun term = new AFun(the_world, "term");
    if (p.isLeafPath()) {
      return 0;
    }
    switch (t.getType()) {
    case ATerm.APPL:
      ATermAppl at = (ATermAppl) t;
      if (at.getFun().equals("appl")) {
	  return areaStartPrime(p.tail(), AsFix.acc(at,args));
      } else if (at.getFun().equals("list")) {
	  return areaStartPrime(p.tail(), AsFix.acc(at,elems));
      } else if (at.getFun().equals("term")) {
	  Path pp = p.tail();
	  pp.decreaseHead();
	  return areaLength(AsFix.acc(at,w_left)) + 
	      areaStartPrime(pp,AsFix.acc(at,term));
      }
    case ATerm.LIST:
      ATermList tl = (ATermList) t;
      if (p.headZero()) {
	return areaStartPrime(p, tl.getATerms().getFirst());
      } else {
	  at = (ATermAppl) tl.getATerms().getFirst();
	  /*	  if (at.getFun().equals("term")) {
	      Path pp = p.tail();
	      pp.decreaseHead();
	      return areaLength(AsFix.acc(at,w_left)) + 
		  areaStartPrime(pp,AsFix.acc(at,term));
	  }
	  */
	  p.decreaseHead();
	  return areaLength(tl.getATerms().getFirst()) + 
	      areaStartPrime(p, new 
			     ATermList(the_world,tl.getATerms().getNext()));
      }
    default:
      System.err.println("Exception in Area.areaStartPrime()");
      System.exit(2);
    }
    return 0;
  }
  
  public static int areaLength(ATerm t) {
    return Annotations.getLn(t);
  }

    /** Is i after the current area?
	@param i the current position
    */
    public boolean after(int i) {
	int start;
	int length;
	start = start();
	length = length();
	if ((start+length) < i) {
	    return true;
	} else {
	    return false;
	}
    }

    /** Is i in the current area?
	@param i the current position
    */
    public boolean in(int i) {
	int start;
	int length;
	start = start();
	length = length();
	if ((start <= i) && (i <= (start+length))) {
	    return true;
	} else {
	    return false;
	}
    }

    /** Returns an ATerm representation of this Area */
    public ATermAppl toATerm() {
	return this;
    }
}
