/*

    Meta-Environment - An environment for language prototyping.
    Copyright (C) 2000  Stichting Mathematisch Centrum, Amsterdam, The Netherlands. 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

    $Id: Focus.java,v 1.7 2000/04/15 15:18:43 mdejonge Exp $
*/
package nl.cwi.gipe.stred;
import toolbus.aterm.*;

public class Focus {
    protected boolean errorSort = false;
    public static final String sym = "focus";

    private Path p;
    private Sort s;
    private Area a;
    private FocusStatus fs;
    // The focus
    // Constructors
    public Focus(Path path, Sort sort, Area area){
	p = path;
	s = sort;
	a = area;
	fs = new FocusStatus();
    }
    // Methods
    // 
    /** Parse the ATerm representation of a focus to an internal Focus object
     *  @param t The ATerm representing a Focus
     *  @return A Focus object representing the same Focus.
     */
    public Focus(ATermAppl t) {
	ATerms args;
	ATerm tmp;

	// We do a do-or-die check on whether this _is_ a focus
	if (sym.equals(t.getFun()) == false) {
	    System.err.println("Malformed term in Focus construction");
	    System.exit(2);
	}
	args = t.getArgs();
	tmp = args.getFirst();
	p = new Path((ATermAppl) tmp);
	args = args.getNext();
	tmp = args.getFirst();
	s = new Sort((ATermAppl) tmp);
	args = args.getNext();
	tmp = args.getFirst();
	a = new Area((ATermAppl) tmp);
	fs = new FocusStatus();
	fs.setStatus(t);
    }
  
    /** Return an ATerm representation of a Focus
     */
    public ATermAppl toATerm() {
	ATermAppl result;
	result = new ATermAppl(sym,new ATerms(p.toATerm(),new ATerms(s.toATerm(), new ATerms(a.toATerm()))));
	fs.setStatusAnno(result);
	return result;
    }

    /** Return a String representation of a Focus */
    public String toString() {
	return toATerm().toString();
    }

    /** Setters and Getters */
    public Path getPath() {
	return p;
    }
    
    public Sort getSort() {
	if (this.errorSort == false) {
	    return s;
	} else {
	    //    throw new ErrorSortException("Focus.sort: error sort occurred");
	    System.err.println("Error sort in Sort.sort()");
	    System.exit(2);
	    return null;
	}
    }

    public Area getArea() {
	return a;
    }

    public void setArea(Area area) {
	a = area;
    }
    public void setPath(Path path) {
	p = path;
    }
    public void setSort(Sort sort) {
	s = sort;
    }
  
    public void setErrorSort() {
  	this.errorSort = true;
    }

    /** Is i after the current focus?
	@param i the current position
    */
    public boolean after(int i) {
	return a.after(i);
    }

    /** Is i in the current focus?
	@param i the current position
    */
    public boolean in(int i) {
	return a.in(i);
    }
  
  //public boolean before(int i) {} apparently is not needed in the implementation :-)

    /** Make this a dirty focus
     */
    public void setDirty() {
	fs.setStatus(false);
    }

    public boolean isClean() {
	return fs.getStatus();
    }

    /** Cheapskate equals() implementation does a string comparison of focus.toString()'s
	@param f The focus that will be tested for equality
    */
    public boolean equals(Object f) {
	return this.toString().equals(f.toString());
    }
}
    
