/*

    Meta-Environment - An environment for language prototyping.
    Copyright (C) 2000  Stichting Mathematisch Centrum, Amsterdam, The Netherlands. 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

    $Id: FocusMove.java,v 1.8 2000/04/15 15:18:43 mdejonge Exp $
*/
package nl.cwi.gipe.stred;
import toolbus.aterm.*;

public class FocusMove {
  public static Editor move(Editor e, Move m) throws MoveException {
      FocusList fl;
      
      fl = e.focuses();
    switch (m.direction()) {
    case Move.UP:
      Path p = fl.head().getPath();
      SeTool.debug("Path: "+p.toString());
      Path p1 = p.up();
      SeTool.debug("Path.up: "+p1.toString());
      try {
	ATerm t = Subtree.subtree(e.tree(),p1);
      }
      catch (SubtreeException se) {
	return e;
      }
      Focus f = Focuses.createFocus(e.tree(), p1);
      fl.tail();
      fl.insertHead(f);
      return e.setFocuses(fl);
    case Move.DOWN:
      Path pd = fl.head().getPath();
      SeTool.debug("Path: "+pd.toString());
      Path pd1 = pd.down();
      SeTool.debug("Path.down: "+pd1.toString());
      try {
	ATerm td = Subtree.subtree(e.tree(),pd1);
	if (td.getType() == ATerm.APPL) {
	  ATermAppl atd = (ATermAppl) td;
	  String dstr = atd.getFun();
	  SeTool.debug("The symbol: "+dstr);
	  if (dstr.equals("appl") || dstr.equals("list") || dstr.equals("lex")) {
	    Focus df = Focuses.createFocus(e.tree(),pd1);
	    fl.tail();
	    fl.insertHead(df);
	    return e.setFocuses(fl);
	  } 
	}
      }
      catch (SubtreeException se) {
	return e;
      }
      // Something went wrong:
      Focus df2 = Focuses.createFocus(e.tree(),pd1);
      df2.setErrorSort();
      fl.tail();
      fl.insertHead(df2);
      return move(e.setFocuses(fl), new Move(Move.RIGHT));
    case Move.RIGHT:
      Path pr = fl.head().getPath();
      SeTool.debug("Path: "+pr.toString());
      Path pr1 = pr.right();
      SeTool.debug("Path.right: "+pr1.toString());
      try {
	ATerm tr = Subtree.subtree(e.tree(),pr1);
	Focus fr = Focuses.createFocus(e.tree(),pr1);
	fl.tail();
	if (tr.getType() == ATerm.APPL) {
	  ATermAppl atr = (ATermAppl) tr;
	  String strr = atr.getFun();
	  SeTool.debug("The function symbol "+strr);
	  if (!(strr.equals("list") || strr.equals("appl") || strr.equals("lex"))) {
	      //	    fr.setErrorSort();
	    fl.insertHead(fr);
	    e.setFocuses(fl);
	    return move(e,new Move(Move.RIGHT));
	  }
	}
	fl.insertHead(fr);
	e.setFocuses(fl);
	return e;
      }
      catch (SubtreeException se) {
	return e;
      }
    case Move.LEFT:
      Path pl = fl.head().getPath();
      SeTool.debug("Path: "+pl.toString());
      if (pl.tailZero()) {
	  return e;
      }
      Path pl1 = pl.left();
      SeTool.debug("Path.left: "+pl1.toString());
      try {
	ATerm tl = Subtree.subtree(e.tree(),pl1);
	Focus fleft = Focuses.createFocus(e.tree(),pl1);
	fl.tail();
	if (tl.getType() == ATerm.APPL) {
	  ATermAppl atl = (ATermAppl) tl;
	  String strl = atl.getFun();
	  if (!(strl.equals("list") || strl.equals("appl") || strl.equals("lex"))) {
	      //	    fleft.setErrorSort();
	      fl.insertHead(fleft);
	      e.setFocuses(fl);
	    return move(e,new Move(Move.LEFT));
	  }
	}
	fl.insertHead(fleft);
	e.setFocuses(fl);
	return e;
      }
      catch (SubtreeException se) {
	return e;
      }
    default:
      throw new MoveException("MoveException in FocusMove.move");
    }
  }
  
  public static Editor toTop(Editor editor) {
  	ATerm tree = editor.tree();
		Focus newFocus = Focuses.createFocus(tree, new Path());
		Editor rootEditor = new Editor(tree, new FocusList(newFocus));
		return rootEditor;
  }
}
