/*

    Meta-Environment - An environment for language prototyping.
    Copyright (C) 2000  Stichting Mathematisch Centrum, Amsterdam, The Netherlands. 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

    $Id: Path.java,v 1.8 2000/04/15 15:18:44 mdejonge Exp $
*/
package nl.cwi.gipe.stred;
import toolbus.aterm.*;

public class Path extends ATermAppl {
  public static final String sym = "path";
  // Constructor
  // path([0]) is only constructor (root)
  public Path(){
    super(the_world, sym,new ATerms(the_world, new ATermInt(the_world,0)));
  }
  //Cast constructor
    public Path(ATermAppl t) {
    //Hmmm. First we call the super class:
    super(the_world, sym,t.getArgs());
    //Then we check whether it was allowed :-) 
    if (sym.equals(t.getFun()) == false) {
      //No further check. Could test whether all elements of list are ints...
      System.err.println("Malformed term in Path constructor");
      System.exit(2);
    }
  }
  
  // Public Methods

  // Predicates
  public boolean isLeafPath(){
    boolean result = false;
    ATerms ts = this.getArgs();
    ATermInt ti = (ATermInt) ts.getFirst();
    int i = ti.getInt();
    if ((ts.length() == 1) && (i == 0)) {
      result = true;
    }
    return result;
  }

  public boolean empty(){
    boolean result = false;
    if (this.getArgs().isEmpty()) {
      result = true;
    }
    return result;
  }

    public boolean targetParent() {
	return (this.getArgs().length() == 2);
    }

    /** Is the last element of this path zero?
     */
    public boolean tailZero() {
	ATerms ts = this.getArgs();
	while (ts.length() > 1) {
	    ts = ts.getNext();
	}
	ATermInt ti = (ATermInt) ts.getFirst();
	return (ti.getInt() == 0);
    }

  public boolean headZero(){
    boolean result = false;
    ATerms ts = this.getArgs();
    ATermInt ti = (ATermInt) ts.getFirst();
    int i = ti.getInt();
    if ((ts.length() > 1) && (i == 0)) {
      result = true;
    }
    return result;
  }

  public boolean isChildPath(Path p){
    boolean result = false;
    if ((p.empty() == false) && (this.head().equals(p.head()))) {
      result = this.tail().isChildPath(p.tail());
    } else if (p.empty()) {
      result = true;
    }
    return result;
  }

  // Path manipulation
  public void decreaseHead(){
    ATerms ts = super.getArgs();
    ATermInt ti1 = (ATermInt) ts.getFirst();
    ATermInt ti = new ATermInt(the_world, ti1.getInt()-1);
    super.setArgs(new ATerms(the_world, ti, ts.getNext()));
  }

  public ATermInt head(){
    return (ATermInt) super.getArgs().getFirst();
  }

  public Path tail(){
    Path p = (Path) this.clone();
    p.setArgs(this.getArgs().getNext());
    return p;
  }

  public Path up() {
    Path p = new Path(this);
    return p.removeLastElement();
  }
  public Path down() {
    //  Path p = (Path) this.clone();
    //debug
    //    System.err.println("In down. this = "+this.toString());
    ATerms ts = this.getArgs();
    ts = ts.concat(new ATerms(the_world,new ATermInt(the_world,0)));
    //    System.err.println("ts = "+ts.toString());
    this.setArgs(ts);
    return this;
  }
  public Path left(){
    return this.decreaseLastElement();
  }
  public Path right(){
    return this.increaseLastElement();
  }

  private Path removeLastElement() {
    Path p;
    switch (super.getArgs().length()) {
    case 1:
      return this;
    case 2:
      p = (Path) this.clone();
      p.setArgs(new ATerms(the_world, p.getArgs().getFirst()));
      return p;
    default:
      p = (Path) this.clone();
      ATermInt ti = (ATermInt) this.getArgs().getFirst();
      p.setArgs(p.getArgs().getNext());
      Path pp = p.removeLastElement();
      pp.setArgs(new ATerms(the_world, ti, pp.getArgs()));
      return pp;
    }
  }
  private Path decreaseLastElement(){
    Path p = (Path) this.clone();
    if (p.getArgs().length() == 1) {
      ATermInt ti = (ATermInt) p.getArgs().getFirst();
      ti.setInt(ti.getInt()-1);
      p.setArgs(new ATerms(the_world, ti));
      return p;
    } else {
      ATermInt ti = (ATermInt) this.getArgs().getFirst();
      p.setArgs(p.getArgs().getNext());
      p = p.decreaseLastElement();
      p.setArgs(new ATerms(the_world, ti, p.getArgs()));
      return p;
    }
  }
  private Path increaseLastElement(){
    Path p = (Path) this.clone();
    if (p.getArgs().length() == 1) {
      ATermInt ti = (ATermInt) p.getArgs().getFirst();
      ti.setInt(ti.getInt()+1);
      p.setArgs(new ATerms(the_world, ti));
      return p;
    } else {
      ATermInt ti = (ATermInt) this.getArgs().getFirst();
      p.setArgs(p.getArgs().getNext());
      p = p.increaseLastElement();
      p.setArgs(new ATerms(the_world, ti, p.getArgs()));
      return p;
    }
  }

    /** Return an ATerm representation of this Path */
    public ATermAppl toATerm() {
	return this;
    }
}
      


