/*

    Meta-Environment - An environment for language prototyping.
    Copyright (C) 2000  Stichting Mathematisch Centrum, Amsterdam, The Netherlands. 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

    $Id: StructureEditTest.java,v 1.1 2000/04/15 15:18:46 mdejonge Exp $
*/
package nl.cwi.gipe.stred.test;
import nl.cwi.gipe.stred.*;
import junit.framework.*;
import java.io.*;
import toolbus.aterm.*;

public class StructureEditTest extends TestCase {

    public StructureEditTest(String str) {
	super(str);
    }

    public void testMakeTreeInvalid() throws IOException, ClassNotFoundException, MoveException {
	ATermTest aterm = new ATermTest();
	Editor editor = StructureEdit.newEditor("");
	
	editor =  StructureEdit.replaceFocus(editor, aterm.picoTerm);

	editor = StructureEdit.makeTreeInvalid(editor);

	/* check if this went ok */
	
	/* there's only one focus */
	assertEquals(editor.focuses().size(),1);
		
	/* The focus should be invalid */
	assertEquals(new Sort("invalid"),editor.focuses().head().getSort());

	/* The focus shoudl be dirty (!clean) */
	assert(!editor.focuses().head().isClean());

	/* The focus should be on the top */
	assertEquals(editor.focuses().head().getArea(), FocusMove.toTop(editor).focuses().head().getArea());

	/* There should be only one node in the tree, but we don't know how to test this */
    }
	
    public void testInsertChars() throws IOException, ClassNotFoundException, MoveException, ParseError, SubtreeException {
	EditorTest editortest = new EditorTest();
	Editor editor = editortest.picoEditor;

	editor = StructureEdit.insertChars(editor, 20, "aaaap!");

	/* check if this went ok */
	
	/* there's only one focus */
	assertEquals(editor.focuses().size(),1);
		
	/* The focus shoudl be dirty (!clean) */
	assert(!editor.focuses().head().isClean());

	/* The focus should be on the top */
	assertEquals(editor.focuses().head().getArea(), FocusMove.toTop(editor).focuses().head().getArea());
	
	/* There should be only one node in the tree, but we don't know how to test this */

    }
    public static Test suite() {
  	TestSuite suite= new TestSuite(StructureEditTest.class);
	return suite;
    }
}
